#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.utils.typehelper import TypeHelper
from cli.messages import Messages


class UpdateCommand(OvirtCommand):

    name = 'update'
    description = 'update an object'
    args_check = 2
    valid_options = [ ('*', str) ]

    helptext = """\
        == Usage ==

        update <type> <id> [parent identifiers] [command options]

        == Description ==

        Update an existing object. This command requires the following
        arguments:

          * type        The type of object to update.
          * id          The identifier of the object to update

        == Supported Help formats ==

        - This help will list all available attribute options for given resource update
          
          * format      - help update type
          * example     - help update vm

        - This help will list all available attribute options for given subresource update
          
          * format      - help update type --parentid
          * example     - help update disk --vm-name myvm

        == Available Types ==

        The following object types are available:

          $types

        == Base Identifiers ==

        Some objects can exist inside other objects. For example, a disk can
        exist in the content of a virtual machine. In this case, one or more
        object identifier options need to be provided to identify the
        containing object.

        An object identifier is an option of the form
        '--parent-<type>-identifier <id>' or '--parent-<type>-name <name>'. This
        would identify an object with type <type> and id <id> or name <name>.
        See the examples section below for a few examples.

        == Attribute Options ==

        Attribute options specifiy values for attributes of the object that is
        to be updated.

        Type 'help update <type>' to see an overview of which attributes are
        available for a specific type.

        == Examples ==

        - This example updates a virtual machine with name "myvm", setting new 
          description and amount of monitors.

          $ update vm myvm --display-monitors 1 --description test1

        - This example updates a virtual machine disk with name "mydisk", setting 
          "bootable" to true.

          $ update disk "mydisk" --parent-vm-name "myvm" --bootable true

        == Return Values ==

          $statuses
        """

    helptext1 = """\
        == Usage ==

        update <type> <id> [parent identifiers] [command options]

        == Description ==

        Update an existing object. This command requires the following
        arguments:

          * type        The type of object to update.
          * id          The identifier of the object to update

        See 'help update' for generic help on creating objects.

        == Attribute Options ==

        The following options are available for objects with type '$type':

          $options

        == Collection based option format ==

          * [--x-y: collection]
            {
              [y.a: string]
              [y.b: string]
              [y.c: string]
            }

          e.g:

          --x-y "y.a=a1,y.b=b1,y.c=c1"
          --x-y "y.a=a2,y.b=b2,y.c=c2"
          ...

          where a, b, c are option properties and aN, bN, cN is actual user's data

        == Return Values ==

          $statuses
        """

    def execute(self):
        """Execute the "update" command."""
        args = self.arguments
        opts = self.options

        typs = TypeHelper.get_types_containing_method(
                      'update',
                       expendNestedTypes=True,
                       groupOptions=True
        )

        resource = self.get_object(
                       args[0], args[1],
                       self.resolve_base(opts),
                       context_variants=typs[args[0]]
        )

        if resource is None:
            self.error(
               Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
            )
        elif hasattr(resource, 'update'):
            obj = self.update_object_data(self.__create_set_superclass(resource), opts)
            result = self.execute_method(obj, 'update', opts)
        else:
            self.error(
               Messages.Error.OBJECT_IS_IMMUTABLE % (args[0], args[1])
            )
        self.context.formatter.format(self.context, result)

    def __create_set_superclass(self, resource):
        """Create an instance of param object."""
        param_obj = type(resource.superclass).factory()
        if hasattr(param_obj, 'id'):
            setattr(param_obj, 'id', getattr(resource, 'id'))
        resource.superclass = param_obj
        return resource

    def show_help(self):
        """Show help for "update"."""

        self.check_connection()
        args = self.arguments
        opts = self.options

        subst = {}
        types = TypeHelper.get_types_containing_method(
               'update',
               expendNestedTypes=True,
               groupOptions=True
        )
        subst['types'] = self.format_map(types)
        statuses = self.get_statuses()
        subst['statuses'] = self.format_list(statuses)

        if len(args) > 0 and self.is_supported_type(types.keys(), args[0]):
            if len(args) == 2:
                base = self.resolve_base(self.options)
                obj = self.get_object(
                          args[0], args[1],
                          base,
                          context_variants=types[args[0]]
                )
                if obj is None:
                    self.error(
                          Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
                    )
                helptext = self.helptext1
                params_list = self.get_options(
                           method='update',
                           resource=obj,
                           sub_resource=base,
                           context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(params_list)
                subst['type'] = args[0]

            elif len(args) == 1 and len(opts) == 2:
                helptext = self.helptext1

                subst['type'] = args[0]

                options = self.get_options(
                           method='update',
                           resource=args[0],
                           sub_resource=self.resolve_base(self.options),
                           context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(options)
                subst['type'] = args[0]
            elif len(args) == 1:
                helptext = self.helptext
                subst['type'] = args[0]
                subst['types'] = self.format_map({args[0]:types[args[0]]})
            else:
                helptext = self.helptext
        else:
            helptext = self.helptext

        helptext = self.format_help(helptext, subst)
        self.write(helptext)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import textwrap
from optparse import OptionParser
from ovirtcli.infrastructure import settings


class OvirtCliOptionParser(OptionParser):

    usage = '%prog [options]\n       %prog [options] command...'
    description = textwrap.dedent("""\
        This program is a command-line interface to %s Virtualization.
        """ % settings.OvirtCliSettings.PRODUCT)

    def __init__(self):

        OptionParser.__init__(self, usage=self.usage,
                              description=self.description)
        self.add_option('-d', '--debug', action='store_true',
                        help='enable debugging')
        self.add_option('-l', '--url',
                        help='specifies the API entry point URL (http[s]://server[:port]/ovirt-engine/api)')
        self.add_option('-u', '--username', help='connect as this user')
        self.add_option('-K', '--key-file', help='specify client PEM key-file')
        self.add_option('-C', '--cert-file', help='specify client PEM cert-file')
        self.add_option('-A', '--ca-file', help='specify server CA cert-file')
        self.add_option('-I', '--insecure', help='allow connecting to SSL sites without CA certificate',
                        action='store_true')
        self.add_option('-r', '--renew-session', help='automatically renew session upon expiration',
                        action='store_true')
        self.add_option('-D', '--dont-validate-cert-chain', help='do not validate server CA certificate',
                        action='store_true')
        self.add_option('-F', '--filter', help='enables user permission based filtering',
                        action='store_true')
        self.add_option('-P', '--port', help='specify port')
        self.add_option('-T', '--timeout', help='specify request timeout')
        self.add_option('-S', '--session-timeout', help='specify authentication session timeout in minutes (positive number)')
        self.add_option('-c', '--connect', action='store_true',
                        help='automatically connect')
        self.add_option('-f', '--file', metavar='FILE',
                        help='read commands from FILE instead of stdin')
        self.add_option('-e', '--extended-prompt', action='store_true',
                        help='display extra information in the prompt')
        self.add_option('-E', '--execute-command',
                        help='execute command and print the output to STDIO')
        self.add_option('--kerberos', action='store_true',
                        help='use Kerberos authentication')
        self.disable_interspersed_args()





#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import sys

from ovirtcli.infrastructure.options import OvirtCliOptionParser
from ovirtcli.infrastructure.context import OvirtCliExecutionContext
from ovirtcli.infrastructure.object import create
from ovirtcli.shell.engineshell import EngineShell

def main():
    # Parse the command line:
    parser = create(OvirtCliOptionParser)
    opts, args = parser.parse_args()

    # Convert the options to a dictionary, so the rest of the code doesn't
    # have to deal with optparse specifics:
    opts = vars(opts)

    # Create the execution context:
    context = OvirtCliExecutionContext(opts=opts, args=args)

    # Create the command interpreter:
    shell = EngineShell(context)
    shell.onecmd_loop()

if __name__ == '__main__':
    main()



#
# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import re


# These are the regular expressions used to check if an option should be
# used to find a parent entity. The preferred way is using the "parent"
# prefix, as it avoid potential conflicts with other options, but we
# need to support the "identifier" suffix without the "parent" prefix
# for backwards compatibility.
PARENT_ID_OPTION_EXPRESSIONS = [
    re.compile(r"^--parent-(?P<type>.+)-(identifier|name)$"),
    re.compile(r"^--(?P<type>.+)-identifier$"),
]


class OptionHelper(object):

    @staticmethod
    def is_parent_id_option(option):
        """
        Checks if the given option name is a reference to a parent
        entity.
        """
        for parent_id_option_expression in PARENT_ID_OPTION_EXPRESSIONS:
            if parent_id_option_expression.match(option):
                return True
        return False

    @staticmethod
    def get_parent_id_type(option):
        """
        Extracts the name of the type from an option that is a reference to
        a parent entity. For example, if the option is "--parent-host-name"
        this method will return "host".
        """
        for parent_id_option_expression in PARENT_ID_OPTION_EXPRESSIONS:
            match = parent_id_option_expression.match(option)
            if match is not None:
                return match.group("type")
        return None



#
# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import os
import re
import tempfile
import urllib

from cli.error import Error
from ovirtcli.platform import util
from cli.messages import Messages


# Template to generate the remote viewer configuration file:
CONFIG_TEMPLATE = u"""\
[virt-viewer]
type=spice
title={title}:%d - Press SHIFT+F12 to Release Cursor
host={host}
port={port}
tls-port={secport}
password={ticket}
fullscreen=0
enable-smartcard=0
enable-usb-autoshare=1
delete-this-file=1
usb-filter=-1,-1,-1,-1,0
tls-ciphers=DEFAULT
host-subject={host_subject}
toggle-fullscreen=shift+f11
release-cursor=shift+f12
secure-attention=ctrl+alt+del
secure-channels=main;inputs;cursor;playback;record;display;usbredir;smartcard
ca={ca_text}
"""


def launch_spice_client(host, host_subject, port, secport, ticket, url,
                        title, proxy, debug=False):
    """Launch the SPICE client."""

    # Check that there is a X display available:
    display = os.environ.get('DISPLAY')
    if display is None:
        raise Error, Messages.Error.CANNOT_START_CONSOLE_CLIENT % 'SPICE'

    # Download and save the CA certificate tot he directory and file where the
    # clients expect it:
    ca_dir = os.path.join(util.get_home_dir(), '.spicec')
    try:
        os.stat(ca_dir)
    except OSError:
        os.mkdir(ca_dir)
    ca_file = os.path.join(ca_dir, 'spice_truststore.pem')
    try:
        os.stat(ca_file)
    except OSError:
        ca_text = download_ca_certificate(url)
        with open(ca_file, "w") as ca_stream:
            ca_stream.write(ca_text)

    # Try to use remote viewer:
    cmd = util.which("remote-viewer")
    if cmd is not None:
        launch_remote_viewer(cmd, host, host_subject, port, secport, ticket,
                             title, ca_file, proxy, debug)
        return

    # Try to use spicec (this is older, so we try only if remote-viewer isn't
    # available):
    cmd = util.which('spicec')
    if cmd is None:
        cmd = util.which('/usr/libexec/spicec')
    if cmd is not None:
        launch_spicec(cmd, host, host_subject, port, secport, ticket,
                      title, ca_file, debug)
        return

    # No luck, no known command is available:
    raise Error, Messages.Error.NO_SPICE_VIEWER_FOUND


def launch_spicec(cmd, host, host_subject, port, secport, ticket,
                  title, ca_file, debug=False):
    """Launch the SPICE client using the spicec command."""

    args = ['spicec']
    if cmd.startswith('/usr/libexec'):
        args.extend([host])
        if port is not None:
            args.append([str(port)])
        else:
            args.extend(["0"])
        if secport:
            args.extend([str(secport)])
            args.extend(['--ssl-channels', 'smain,sinputs'])
            args.extend(['--ca-file', ca_file])
            if host_subject and host_subject != '':
                args.extend(['--host-subject', host_subject])
        args.extend(['-p', ticket])
    else:
        args.extend(['-h', host])
        if port is not None:
            args.extend(['-p', str(port)])
        if secport:
            args.extend([ '-s', str(secport) ])
            args.extend(['--ca-file', ca_file])
            if host_subject and host_subject != '':
                args.extend(['--host-subject', host_subject])
        args.extend(['-w', ticket])
        args.extend(['-t', title])
    util.spawn(cmd, args, debug=True)


def launch_remote_viewer(cmd, host, host_subject, port, secport, ticket,
                         title, ca_file, proxy, debug=False):
    """Launch the SPICE client using the remote-viewer command."""

    # Load the CA certificate:
    with open(ca_file, "r") as ca_stream:
        ca_text = ca_stream.read()

    # Generate the configuration:
    config_text = CONFIG_TEMPLATE.format(
        title=title,
        host=host,
        ticket=ticket,
        port=port,
        secport=secport,
        host_subject=host_subject,
        ca_text=ca_text.replace("\n", "\\n"),
    )

    # Add the proxy:
    if proxy is not None:
        config_text += "proxy=%s\n" % proxy

    # Write the configuation to a temporary file:
    config_fd, config_path = tempfile.mkstemp()
    with os.fdopen(config_fd, "w") as config_stream:
        config_stream.write(config_text.encode("utf-8"))

    # Run the remote-viewwer command:
    args = ["remote-viewer", config_path]
    util.spawn(cmd, args, debug)


def download_ca_certificate(url):
    """Downloads the CA certificate from the engine."""

    ca_url = re.sub("^https?://([^/]+)/.*", "http://\\1/ca.crt", url)
    ca_file = None
    try:
        ca_fd, ca_file = tempfile.mkstemp()
        urllib.urlretrieve(ca_url, ca_file)
        with os.fdopen(ca_fd, "r") as ca_stream:
            ca_text = ca_stream.read()
        return ca_text
    finally:
        if ca_file is not None:
            os.remove(ca_file)




import os
import sys

from distutils.command.build import build
from distutils.command.build_py import build_py
from setuptools import setup, Command


version_info = {
    'name': 'ovirt-shell',
    'version': '3.6.7.0',
    'description': 'A command-line interface to oVirt Virtualization',
    'author': 'Michael Pasternak',
    'author_email': 'mpastern@redhat.com',
    'url': 'http://www.ovirt.org/wiki/CLI',
    'license': 'ASL2',
    'classifiers': [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.4' ],
}

class custom_build_py(build_py):
    """
    This custom build implementation is needed in order to generate
    the parsing and lexing tables used by ply during buildtime, so
    that it won't attempt to generate them during runtime.
    """
    def run(self):
        build_py.run(self)
        if not self.dry_run:
            self.generate_parsing_tables()

    def generate_parsing_tables(self):
        """
        Generates the parsing tables used by ply.
        """
        print("generating parsing tables")
        old_path = sys.path
        try:
            sys.path = [self.build_lib] + sys.path
            import cli.parser
            parser = cli.parser.Parser()
            parser._write_tables()
        finally:
            sys.path = old_path

setup(
    package_dir={ '': 'src' },
    packages=[ 'ovirtcli', 'ovirtcli.command', 'ovirtcli.format',
                 'ovirtcli.platform', 'ovirtcli.platform.posix',
                 'ovirtcli.platform.windows', 'ovirtcli.shell', 'ovirtcli.utils', 'cli',
                 'cli.command', 'cli.platform', 'cli.platform.posix', 'ovirtcli.infrastructure',
                 'ovirtcli.annotations', 'ovirtcli.events', 'ovirtcli.listeners', 'ovirtcli.meta',
                 'ovirtcli.state'],
    install_requires=[ 'ovirt-engine-sdk-python >= 3.6.7.0', 'ply >= 3.3', 'kitchen >= 1' ],
    entry_points={ 'console_scripts': [ 'ovirt-shell = ovirtcli.main:main' ] },
    cmdclass={
        "build_py": custom_build_py,
    },
    **version_info
)



#
# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import os
import tempfile

from cli.error import Error
from cli.messages import Messages
from ovirtcli.platform import util
from subprocess import Popen, PIPE


# Template to generate the remote viewer configuration file:
CONFIG_TEMPLATE = u"""\
[virt-viewer]
type=vnc
host={host}
port={port}
password={ticket}
delete-this-file=1
title={title}:%d - Press SHIFT+F12 to Release Cursor
toggle-fullscreen=shift+f11
release-cursor=shift+f12
secure-attention=ctrl+alt+end
"""


def launch_vnc_client(host, port, ticket, title, debug=False):
    """Launch the VNC client."""

    # Check that there is a X display available:
    display = os.environ.get('DISPLAY')
    if display is None:
        raise Error, Messages.Error.CANNOT_START_CONSOLE_CLIENT % 'VNC'

    # Try to use remote viewer:
    cmd = util.which("remote-viewer")
    if cmd is not None:
        launch_remote_viewer(cmd, host, port, ticket, title, debug)
        return

    # Try to use vncviewer:
    cmd = util.which("vncviewer")
    if cmd is not None:
        launch_vncviewer(cmd, host, port, ticket, debug)
        return

    # No luck, no known command is available:
    raise Error, Messages.Error.NO_VNC_VIEWER_FOUND


def launch_vncviewer(cmd, host, port, ticket, debug=False):
    """Launch a VNC viewer on host::port with `password'."""

    cmd_passwd = util.which('vncpasswd')
    if cmd_passwd is None:
        raise Error, Messages.Error.NO_SUCH_COMMAND % 'vncpasswd'
    p = Popen([cmd_passwd, "-f"], shell=False, stdin=PIPE, stdout=PIPE)
    password = p.communicate(input=ticket)[0]
    args = [cmd, '%s::%s' % (host, port), '-passwordFile', '/dev/stdin' ]
    pid, pstdin = util.spawn(cmd, args, debug)
    os.write(pstdin, password)
    os.close(pstdin)


def launch_remote_viewer(cmd, host, port, ticket, title, debug=False):
    """Launch the VNC client using the remote-viewer command."""

    # Generate the configuration file:
    config_text = CONFIG_TEMPLATE.format(
        title=title,
        host=host,
        ticket=ticket,
        port=port,
    )
    config_fd, config_path = tempfile.mkstemp()
    with os.fdopen(config_fd, "w") as config_stream:
        config_stream.write(config_text.encode("utf-8"))

    # Run the remote-viewwer command:
    args = ["remote-viewer", config_path]
    util.spawn(cmd, args, debug)
